"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const uuid = require("uuid/v4");
const amqp = require("amqplib");
const Bluebird = require("bluebird");
const di_1 = require("../di");
const constant_1 = require("../interface/constant");
const logger_1 = require("../util/logger");
const error_1 = require("../util/error");
class MessageDust {
    constructor(serviceName) {
        this.serviceName = serviceName;
        this.responses = {};
        this.workerQueue = `${this.serviceName}`;
        this.fanoutQueue = `${this.serviceName}.${uuid()}`;
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            logger_1.logger.info('[MessageDust] Initializing...');
            try {
                yield this.setupMQ();
            }
            catch (err) {
                logger_1.logger.error('[MessageDust] Failed to initialize: ', err);
                process.exit(1);
            }
        });
    }
    setupMQ() {
        return __awaiter(this, void 0, void 0, function* () {
            const MQ_URL = process.env.MQ_URL || 'rabbitmq';
            const MQ_PORT = process.env.MQ_PORT || '5672';
            this.conn = yield amqp.connect(`amqp://star:devenv@${MQ_URL}:${MQ_PORT}`);
            // TODO: support channel pool
            this.rpcChannel = yield this.conn.createChannel();
            this.rpcChannel.prefetch(1);
            this.eventChannel = yield this.conn.createChannel();
            this.eventChannel.prefetch(1);
            try {
                yield this.eventChannel.assertExchange(constant_1.EXCHANGE, 'topic', { durable: true });
                yield this.eventChannel.assertQueue(this.workerQueue, { autoDelete: true });
                yield this.eventChannel.assertQueue(this.fanoutQueue, { autoDelete: true });
            }
            catch (e) {
                yield this.eventChannel.deleteQueue(this.workerQueue);
                yield this.eventChannel.deleteQueue(this.fanoutQueue);
                throw e;
            }
        });
    }
    closeQ() {
        return __awaiter(this, void 0, void 0, function* () {
            logger_1.logger.info('[MessageDust] Close channels and connection...');
            yield this.eventChannel.close();
            yield this.rpcChannel.close();
            yield this.conn.close();
        });
    }
    destroy() {
        return __awaiter(this, void 0, void 0, function* () {
            logger_1.logger.info('[MessageDust] Destroying...');
            try {
                yield this.closeQ();
            }
            catch (err) {
                logger_1.logger.error('[MessageDust] Failed to destory...\n', err);
            }
        });
    }
    consumeRest(controllers) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const rpcName of Object.keys(controllers)) {
                yield this.rpcChannel.assertQueue(rpcName, { durable: true });
                this.rpcChannel.consume(rpcName, (msg) => {
                    return Bluebird.try(() => {
                        const payload = JSON.parse(msg.content.toString());
                        logger_1.logger.debug(`[MessageDust][REST][${rpcName}][REQ] ${JSON.stringify(payload)} => ${msg.properties.replyTo}`);
                        const controllerClass = controllers[rpcName];
                        const controller = di_1.Di.container.get(controllerClass.clazz);
                        return controller[controllerClass.funcName](payload);
                    }).then((result) => {
                        logger_1.logger.debug(`[MessageDust][REST][${rpcName}][RES] ${JSON.stringify(result)} => ${msg.properties.replyTo}`);
                        this.rpcChannel.sendToQueue(msg.properties.replyTo, new Buffer(JSON.stringify({ success: true, data: result })), { correlationId: msg.properties.correlationId, persistent: true });
                    }).catch((err) => {
                        logger_1.logger.error(err);
                        this.rpcChannel.sendToQueue(msg.properties.replyTo, new Buffer(JSON.stringify({
                            success: false,
                            code: err.code,
                            message: err.toString()
                        })), { correlationId: msg.properties.correlationId, persistent: true });
                    }).finally(() => {
                        this.rpcChannel.ack(msg);
                    });
                });
            }
        });
    }
    consumeRPC(controllers) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const rpcName of Object.keys(controllers)) {
                yield this.rpcChannel.assertQueue(rpcName, { durable: true });
                this.rpcChannel.consume(rpcName, (msg) => {
                    return Bluebird.try(() => {
                        const payload = JSON.parse(msg.content.toString());
                        logger_1.logger.debug(`[MessageDust][RPC][${rpcName}][REQ] ${JSON.stringify(payload)} => ${msg.properties.replyTo}`);
                        const controllerClass = controllers[rpcName];
                        const controller = di_1.Di.container.get(controllerClass.clazz);
                        return controller[controllerClass.funcName](payload);
                    }).then((result) => {
                        logger_1.logger.debug(`[MessageDust][RPC][${rpcName}][RES] ${JSON.stringify(result)} => ${msg.properties.replyTo}`);
                        this.rpcChannel.sendToQueue(msg.properties.replyTo, new Buffer(JSON.stringify({ success: true, data: result })), { correlationId: msg.properties.correlationId, persistent: true });
                    }).catch((err) => {
                        logger_1.logger.error(err);
                        this.rpcChannel.sendToQueue(msg.properties.replyTo, new Buffer(JSON.stringify({
                            success: false,
                            code: err.code,
                            message: err.toString()
                        })), { correlationId: msg.properties.correlationId, persistent: true });
                    }).finally(() => {
                        this.rpcChannel.ack(msg);
                    });
                });
            }
        });
    }
    consumeGraph(controllers) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const rpcName of Object.keys(controllers)) {
                yield this.rpcChannel.assertQueue(rpcName, { durable: true });
                this.rpcChannel.consume(rpcName, (msg) => {
                    return Bluebird.try(() => {
                        const payload = JSON.parse(msg.content.toString());
                        logger_1.logger.debug(`[MessageDust][Graph][${rpcName}][REQ] ${JSON.stringify(payload)} => ${msg.properties.replyTo}`);
                        const controllerClass = controllers[rpcName];
                        const controller = di_1.Di.container.get(controllerClass.clazz);
                        return controller[controllerClass.funcName](payload);
                    }).then((result) => {
                        logger_1.logger.debug(`[MessageDust][Graph][${rpcName}][RES] ${JSON.stringify(result)} => ${msg.properties.replyTo}`);
                        this.rpcChannel.sendToQueue(msg.properties.replyTo, new Buffer(JSON.stringify({ success: true, data: result })), { correlationId: msg.properties.correlationId, persistent: true });
                    }).catch((err) => {
                        logger_1.logger.error(err);
                        this.rpcChannel.sendToQueue(msg.properties.replyTo, new Buffer(JSON.stringify({
                            success: false,
                            code: err.code,
                            message: err.toString()
                        })), { correlationId: msg.properties.correlationId, persistent: true });
                    }).finally(() => {
                        this.rpcChannel.ack(msg);
                    });
                });
            }
        });
    }
    getEventKey(cronEvents, key) {
        const keyParts = key.split('.').slice(1);
        for (const cronEvent of cronEvents) {
            const cronParts = cronEvent.split('.').slice(1);
            let isMatched = true;
            for (const parts of _.zip(cronParts, keyParts)) {
                if (parts[0] !== '*' && parts[0] !== parts[1]) {
                    isMatched = false;
                }
            }
            if (isMatched)
                return cronEvent;
        }
        return '';
    }
    consumeWorkerEvent(controllers) {
        return __awaiter(this, void 0, void 0, function* () {
            const cronEvents = [];
            for (const eventName of Object.keys(controllers)) {
                yield this.eventChannel.bindQueue(this.workerQueue, constant_1.EXCHANGE, eventName);
                if (eventName.startsWith('cron.'))
                    cronEvents.push(eventName);
            }
            this.eventChannel.consume(this.workerQueue, (msg) => {
                return Bluebird.try(() => {
                    const payload = JSON.parse(msg.content.toString());
                    logger_1.logger.debug(`[MessageDust][Worker Event@${payload.key}][REQ] ${JSON.stringify(payload)}`);
                    const eventKey = payload.key.startsWith('cron.') ? this.getEventKey(cronEvents, payload.key) : payload.key;
                    if (!eventKey)
                        throw new error_1.FatalError(error_1.ErrorCode.FATAL.UNREGISTERED_EVENT_KEY);
                    const controllerClass = controllers[eventKey];
                    const controller = di_1.Di.container.get(controllerClass.clazz);
                    return controller[controllerClass.funcName](payload);
                }).catch((err) => {
                    logger_1.logger.error(err);
                }).finally(() => {
                    this.eventChannel.ack(msg);
                });
            });
        });
    }
    consumeFanoutEvent(controllers) {
        return __awaiter(this, void 0, void 0, function* () {
            const cronEvents = [];
            for (const eventName of Object.keys(controllers)) {
                yield this.eventChannel.bindQueue(this.fanoutQueue, constant_1.EXCHANGE, eventName);
                if (eventName.startsWith('cron.'))
                    cronEvents.push(eventName);
            }
            this.eventChannel.consume(this.fanoutQueue, (msg) => {
                return Bluebird.try(() => {
                    const payload = JSON.parse(msg.content.toString());
                    logger_1.logger.debug(`[MessageDust][Fanout Event@${payload.key}][REQ] ${JSON.stringify(payload)}`);
                    const eventKey = payload.key.startsWith('cron.') ? this.getEventKey(cronEvents, payload.key) : payload.key;
                    if (!eventKey)
                        throw new error_1.FatalError(error_1.ErrorCode.FATAL.UNREGISTERED_EVENT_KEY);
                    const controllerClass = controllers[eventKey];
                    const controller = di_1.Di.container.get(controllerClass.clazz);
                    return controller[controllerClass.funcName](payload);
                }).catch((err) => {
                    logger_1.logger.error(err);
                }).finally(() => {
                    this.eventChannel.ack(msg);
                });
            });
        });
    }
    invokeRPC(rpcParam) {
        return __awaiter(this, void 0, void 0, function* () {
            logger_1.logger.debug(`[MessageDust] Invoke RPC with Param: ${JSON.stringify(rpcParam)}`);
            const q = yield this.rpcChannel.assertQueue('', { exclusive: true, autoDelete: true });
            const corrId = uuid().toString();
            this.rpcChannel.consume(q.queue, (msg) => {
                if (msg.properties.correlationId === corrId) {
                    const waiting = this.responses[corrId];
                    if (!waiting) {
                        logger_1.logger.error(`[MessageDust] Got an unknown response: ${corrId}`);
                        return;
                    }
                    delete this.responses[corrId];
                    this.rpcChannel.ack(msg);
                    return waiting.resolve(msg);
                }
            });
            logger_1.logger.debug(`[MessageDust][${corrId}][${rpcParam.uri}] send request...`);
            yield this.rpcChannel.sendToQueue(rpcParam.uri, new Buffer(JSON.stringify(rpcParam.payload)), { correlationId: corrId, replyTo: q.queue, persistent: true });
            logger_1.logger.debug(`[MessageDust][${corrId}][${rpcParam.uri}] Watiting for response...`);
            return new Bluebird((resolve, reject) => {
                this.responses[corrId] = { resolve, reject };
            }).then((msg) => {
                const content = JSON.parse(msg.content.toString());
                logger_1.logger.debug(`[MessageDust][${corrId}][${rpcParam.uri}][RES] ${JSON.stringify(content)}`);
                return content;
            }).timeout(constant_1.Rpc.RPC_TIMEOUT).catch((err) => {
                delete this.responses[corrId];
                throw new error_1.LogicError(error_1.ErrorCode.Logic.RPC_TIMEOUT);
            });
        });
    }
    publishEvent(event) {
        return __awaiter(this, void 0, void 0, function* () {
            const msg = JSON.stringify(event);
            logger_1.logger.debug(`[MessageDust] Publish event: ${msg}`);
            this.eventChannel.publish(constant_1.EXCHANGE, event.key, new Buffer(msg));
            return event.root;
        });
    }
}
exports.MessageDust = MessageDust;
//# sourceMappingURL=message.dust.js.map