"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const _ = require("lodash");
const Consul = require("consul");
const mongo_dust_1 = require("./dust/mongo.dust");
const message_dust_1 = require("./dust/message.dust");
const constant_1 = require("./interface/constant");
const metadata_1 = require("./interface/metadata");
const di_1 = require("./di");
const logger_1 = require("./util/logger");
const error_1 = require("./util/error");
var MetadataKey = metadata_1.Metadata.Key;
class Star {
    constructor(serviceName) {
        this.serviceName = serviceName;
        this.controllerClasses = {};
        this.controllers = {};
        this.typeDefs = [];
        for (const key of metadata_1.Metadata.Keys) {
            this.controllerClasses[key] = [];
            this.controllers[key] = {};
        }
        this.messageDust = new message_dust_1.MessageDust(this.serviceName);
        this.mongoDust = new mongo_dust_1.MongoDust();
    }
    setupConsul() {
        const CONSUL_TOKEN = process.env.CONSUL_TOKEN;
        if (!CONSUL_TOKEN)
            throw new error_1.FatalError(error_1.ErrorCode.FATAL.MISSING_CONSUL_TOKEN);
        this.consul = Consul({
            host: process.env.CONSUL_URL || 'consul',
            port: process.env.CONSUL_PORT || '8500',
            defaults: {
                dc: 'dc1',
                token: CONSUL_TOKEN
            },
            promisify: true
        });
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            logger_1.logger.info('[Star] Initialize app...');
            process.once('SIGTERM', this.destroy.bind(this));
            process.once('SIGINT', this.destroy.bind(this));
            try {
                this.setupConsul();
                yield this.messageDust.initialize();
                yield this.mongoDust.initialize();
            }
            catch (err) {
                logger_1.logger.error('[Star] Failed to initialize: ', err);
                process.exit(1);
            }
            logger_1.logger.info('[Star] Post-initialize app...');
            try {
                yield this.postInitialize();
            }
            catch (err) {
                logger_1.logger.error('[Star] Failed to post-initialize: ', err);
                process.exit(1);
            }
        });
    }
    destroyConsul() {
        return __awaiter(this, void 0, void 0, function* () {
            logger_1.logger.info('[Star] Destorying star from consul...');
            let kvData = yield this.consul.kv.get(`${constant_1.Rpc.STAR_COUNT}/${this.serviceName}`);
            if (!kvData)
                return;
            kvData = Number(kvData.Value);
            if (kvData < 2) {
                yield this.consul.kv.del(`${constant_1.Rpc.STAR_COUNT}/${this.serviceName}`);
                yield this.consul.kv.del({
                    key: `watch/${MetadataKey.Rest}/${this.serviceName}`,
                    recurse: true
                });
                yield this.consul.kv.del({
                    key: `watch/${MetadataKey.Rpc}/${this.serviceName}`,
                    recurse: true
                });
            }
            else {
                kvData -= 1;
                yield this.consul.kv.set(`${constant_1.Rpc.STAR_COUNT}/${this.serviceName}`, kvData.toString());
            }
        });
    }
    destroy() {
        return __awaiter(this, void 0, void 0, function* () {
            logger_1.logger.info('[Star] Destroying...');
            try {
                yield this.destroyConsul();
                yield this.messageDust.destroy();
                yield this.mongoDust.destroy();
            }
            catch (err) {
                logger_1.logger.error('[Star] Failed to destory...\n', err);
            }
            logger_1.logger.info('[Star] Post-destroying...');
            try {
                yield this.postDestroy();
            }
            catch (err) {
                logger_1.logger.error('[Star] Failed to post-destory...\n', err);
                process.exit(1);
            }
            process.exit(0);
        });
    }
    registerStar() {
        return __awaiter(this, void 0, void 0, function* () {
            let kvData = yield this.consul.kv.get(`${constant_1.Rpc.STAR_COUNT}/${this.serviceName}`);
            if (!kvData) {
                kvData = 1;
            }
            else {
                kvData = Number(kvData.Value);
                kvData += 1;
            }
            yield this.consul.kv.set(`${constant_1.Rpc.STAR_COUNT}/${this.serviceName}`, kvData.toString());
        });
    }
    registerRestEndpoints() {
        return __awaiter(this, void 0, void 0, function* () {
            logger_1.logger.info('[Star] Register rest endpoints.');
            const consul = this.consul;
            const controllers = this.controllers[MetadataKey.Rest];
            for (const controllerClass of this.controllerClasses[MetadataKey.Rest]) {
                const endpoints = Reflect.getOwnMetadata(MetadataKey.Rest, controllerClass) || [];
                logger_1.logger.info(`[Star] Registering rest endpoints to consul.kv: ${endpoints.length}`);
                for (const endpoint of endpoints) {
                    const method = endpoint.method;
                    const uri = _.replace(endpoint.uri, /\//g, '|');
                    const key = `${method}@${uri}`;
                    if (!method || !uri) {
                        throw new error_1.FatalError(error_1.ErrorCode.FATAL.NOT_INIT_REST_ENDPOINT);
                    }
                    controllers[key] = { clazz: controllerClass, funcName: endpoint.funcName };
                    yield consul.kv.set(`watch/${MetadataKey.Rest}/${this.serviceName}/${key}`, JSON.stringify(endpoint.context));
                }
            }
        });
    }
    registerRPCEndpoints() {
        return __awaiter(this, void 0, void 0, function* () {
            logger_1.logger.info('[Star] Register rpc endpoints.');
            const consul = this.consul;
            const controllers = this.controllers[MetadataKey.Rpc];
            for (const controllerClass of this.controllerClasses[MetadataKey.Rpc]) {
                const endpoints = Reflect.getOwnMetadata(MetadataKey.Rpc, controllerClass) || [];
                logger_1.logger.info(`[Star] Registering rpc endpoints to consul.kv: ${endpoints.length}`);
                for (const endpoint of endpoints) {
                    const key = `RPC@${endpoint.funcName}`;
                    controllers[key] = { clazz: controllerClass, funcName: endpoint.funcName };
                    yield consul.kv.set(`watch/${MetadataKey.Rpc}/${this.serviceName}/${key}`, JSON.stringify(endpoint.context));
                }
            }
        });
    }
    registerGraphTypeDefs() {
        return __awaiter(this, void 0, void 0, function* () {
            logger_1.logger.info('[Star] Register graph typedefs.');
            const consul = this.consul;
            const typeDefs = this.typeDefs;
            logger_1.logger.info(`[Star] Registering graph typedefs to consul.kv: ${typeDefs.length}`);
            yield consul.kv.set(`graph/${MetadataKey.Graph}/${this.serviceName}/${MetadataKey.TypeDef}`, JSON.stringify(typeDefs.join('\n')));
        });
    }
    registerGraphResolvers() {
        return __awaiter(this, void 0, void 0, function* () {
            logger_1.logger.info('[Star] Register graph endpoints.');
            const consul = this.consul;
            const controllers = this.controllers[MetadataKey.Graph];
            for (const controllerClass of this.controllerClasses[MetadataKey.Graph]) {
                const endpoints = Reflect.getOwnMetadata(MetadataKey.Graph, controllerClass) || [];
                logger_1.logger.info(`[Star] Registering graph endpoints to consul.kv: ${endpoints.length}`);
                for (const endpoint of endpoints) {
                    const key = `Graph@${endpoint.funcName}`;
                    controllers[key] = { clazz: controllerClass, funcName: endpoint.funcName };
                    yield consul.kv.set(`graph/${MetadataKey.Graph}/${this.serviceName}/${key}`, JSON.stringify(endpoint.context));
                }
            }
        });
    }
    registerWorkerEventEndpoints() {
        return __awaiter(this, void 0, void 0, function* () {
            logger_1.logger.info('[Star] Register worker event endpoints.');
            const controllers = this.controllers[MetadataKey.WorkerEvent];
            for (const controllerClass of this.controllerClasses[MetadataKey.WorkerEvent]) {
                const endpoints = Reflect.getOwnMetadata(MetadataKey.WorkerEvent, controllerClass) || [];
                logger_1.logger.info(`[Star] Registering worker event endpoints: ${endpoints.length}`);
                for (const endpoint of endpoints) {
                    const { key, funcName } = endpoint;
                    controllers[key] = { clazz: controllerClass, funcName: funcName };
                }
            }
        });
    }
    registerFanoutEventEndpoints() {
        return __awaiter(this, void 0, void 0, function* () {
            logger_1.logger.info('[Star] Register fanout event endpoints.');
            const controllers = this.controllers[MetadataKey.FanoutEvent];
            for (const controllerClass of this.controllerClasses[MetadataKey.FanoutEvent]) {
                const endpoints = Reflect.getOwnMetadata(MetadataKey.FanoutEvent, controllerClass) || [];
                logger_1.logger.info(`[Star] Registering fanout event endpoints: ${endpoints.length}`);
                for (const endpoint of endpoints) {
                    const { key, funcName } = endpoint;
                    controllers[key] = { clazz: controllerClass, funcName: funcName };
                }
            }
        });
    }
    // For overriding
    postInitialize() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    postDestroy() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.initialize();
                di_1.Di.bindConstant(this.mongoDust.constructor, this.mongoDust);
                di_1.Di.bindConstant(this.messageDust.constructor, this.messageDust);
                yield this.registerStar();
                yield this.registerRestEndpoints();
                yield this.messageDust.consumeRest(this.controllers[MetadataKey.Rest]);
                yield this.registerRPCEndpoints();
                yield this.messageDust.consumeRPC(this.controllers[MetadataKey.Rpc]);
                yield this.registerGraphTypeDefs();
                yield this.registerGraphResolvers();
                yield this.messageDust.consumeGraph(this.controllers[MetadataKey.Graph]);
                yield this.registerWorkerEventEndpoints();
                yield this.messageDust.consumeWorkerEvent(this.controllers[MetadataKey.WorkerEvent]);
                yield this.registerFanoutEventEndpoints();
                yield this.messageDust.consumeFanoutEvent(this.controllers[MetadataKey.FanoutEvent]);
            }
            catch (err) {
                logger_1.logger.error('[Star] Failed to run...\n', err);
                process.exit(1);
            }
        });
    }
    registerRestController(controllerClass) {
        const controllerClasses = this.controllerClasses[MetadataKey.Rest];
        if (controllerClasses)
            controllerClasses.push(controllerClass);
    }
    registerRpcController(controllerClass) {
        const controllerClasses = this.controllerClasses[MetadataKey.Rpc];
        if (controllerClasses)
            controllerClasses.push(controllerClass);
    }
    registerGraphController(controllerClass) {
        const controllerClasses = this.controllerClasses[MetadataKey.Graph];
        if (controllerClasses)
            controllerClasses.push(controllerClass);
    }
    registerGraphTypeDef(typeDef) {
        this.typeDefs.push(typeDef);
    }
    registerWorkerEventController(controllerClass) {
        const controllerClasses = this.controllerClasses[MetadataKey.WorkerEvent];
        if (controllerClasses)
            controllerClasses.push(controllerClass);
    }
    registerFanoutEventController(controllerClass) {
        const controllerClasses = this.controllerClasses[MetadataKey.FanoutEvent];
        if (controllerClasses)
            controllerClasses.push(controllerClass);
    }
}
exports.Star = Star;
//# sourceMappingURL=star.js.map